@@ -1,16 +1,17 @@
 """
 Implement clean_lat_long function
 """
-# pylint: disable=too-many-boolean-expressions
 import re
+from operator import itemgetter
 from typing import Any, Optional, Tuple, Union
 
-import dask.dataframe as dd
 import dask
+import dask.dataframe as dd
 import numpy as np
 import pandas as pd
 
-from .utils import NULL_VALUES, create_report, to_dask
+from ..eda.progress_bar import ProgressBar
+from .utils import NULL_VALUES, create_report_new, to_dask
 
 LAT_LONG_PATTERN = re.compile(
     r"""
@@ -75,12 +76,10 @@ LONG_PATTERN = re.compile(
     re.VERBOSE | re.UNICODE,
 )
 
-STATS = {"cleaned": 0, "null": 0, "unknown": 0}
-
 
 def clean_lat_long(
     df: Union[pd.DataFrame, dd.DataFrame],
-    lat_long_col: Optional[str] = None,
+    lat_long: Optional[str] = None,
     *,
     lat_col: Optional[str] = None,
     long_col: Optional[str] = None,
@@ -89,6 +88,7 @@ def clean_lat_long(
     inplace: bool = False,
     report: bool = True,
     errors: str = "coerce",
+    progress: bool = True,
 ) -> pd.DataFrame:
     """
     This function cleans latitdinal and longitudinal coordinates
@@ -97,13 +97,13 @@ def clean_lat_long(
     ----------
     df
         pandas or Dask DataFrame
-    lat_long_col
+    lat_long
         column name containing latitudinal and longitudinal coordinates
     lat_col
-        column name containing latitudinal coordinates. If specified, lat_long_col
+        column name containing latitudinal coordinates. If specified, lat_long
         must be None
     long_col
-        column name containing longitudinal coordinates. If specified, lat_long_col
+        column name containing longitudinal coordinates. If specified, lat_long
         must be None
     output_format
         the desired format of the coordinates: decimal degrees ("dd"),
@@ -121,78 +121,96 @@ def clean_lat_long(
         * If ‘raise’, then invalid parsing will raise an exception.
         * If ‘coerce’, then invalid parsing will be set as NaT.
         * If ‘ignore’, then invalid parsing will return the input.
+    progress
+        If True, enable the progress bar
     """
-    # pylint: disable=too-many-arguments,too-many-branches
-    reset_stats()
+    # pylint: disable=too-many-branches
 
-    if lat_long_col and (lat_col or long_col):
-        raise ValueError("lat_long_col must be None if either lat_col or long_col is not None")
+    if lat_long and (lat_col or long_col):
+        raise ValueError("lat_long must be None if either lat_col or long_col is not None")
 
     if output_format not in {"dd", "ddh", "dm", "dms"}:
         raise ValueError(
-            f'output_format {output_format} is invalid, it needs to be "dd", "ddh", "dm", or "dms"'
+            f'output_format {output_format} is invalid, it must be "dd", "ddh", "dm", or "dms"'
         )
 
+    # convert to dask
     df = to_dask(df)
-    # specify the metadata for dask apply
-    if lat_long_col:
-        meta = df.dtypes.to_dict()
-        if split:
-            if output_format == "dd":
-                meta.update(zip(("latitude", "longitude"), (float, float)))
-            else:
-                meta.update(zip(("latitude", "longitude"), (str, str)))
-        else:
-            meta[f"{lat_long_col}_clean"] = float if output_format == "dd" else str
 
-        df = df.apply(
-            format_lat_long,
-            args=(lat_long_col, output_format, split, errors),
-            axis=1,
-            meta=meta,
+    # To clean, create a new column "clean_code_tup" which contains
+    # the cleaned values and code indicating how the initial value was
+    # changed in a tuple. Then split the column of tuples and count the
+    # amount of different codes to produce the report
+    if lat_long:
+        # clean a latitude and longitude column
+        df["clean_code_tup"] = df[lat_long].map_partitions(
+            lambda srs: [_format_lat_long(x, output_format, split, errors) for x in srs],
+            meta=object,
         )
+        if split:
+            df = df.assign(
+                latitude=df["clean_code_tup"].map(itemgetter(0)),
+                longitude=df["clean_code_tup"].map(itemgetter(1)),
+                _code_=df["clean_code_tup"].map(itemgetter(2)),
+            )
+        else:
+            df = df.assign(
+                _temp_=df["clean_code_tup"].map(itemgetter(0)),
+                _code_=df["clean_code_tup"].map(itemgetter(1)),
+            )
+            df = df.rename(columns={"_temp_": f"{lat_long}_clean"})
         if inplace:
-            df = df.drop(columns=[lat_long_col])
+            df = df.drop(columns=lat_long)
     else:
         # clean a latitude column
         if lat_col:
-            meta = df.dtypes.to_dict()
-            meta[f"{lat_col}_clean"] = float if output_format == "dd" else str
-            df = df.apply(
-                format_lat_or_long,
-                args=(lat_col, output_format, errors, "lat"),
-                axis=1,
-                meta=meta,
+            df["clean_code_tup"] = df[lat_col].map_partitions(
+                lambda srs: [_format_lat_or_long(x, output_format, errors, "lat") for x in srs],
+                meta=object,
             )
+            df = df.assign(
+                _temp_=df["clean_code_tup"].map(itemgetter(0)),
+                _code_=df["clean_code_tup"].map(itemgetter(1)),
+            )
+            df = df.rename(columns={"_temp_": f"{lat_col}_clean"})
             if inplace:
-                df = df.drop(columns=[lat_col])
+                df = df.drop(columns=lat_col)
         # clean a longitude column
         if long_col:
-            meta = df.dtypes.to_dict()
-            meta[f"{long_col}_clean"] = float if output_format == "dd" else str
-            df = df.apply(
-                format_lat_or_long,
-                args=(long_col, output_format, errors, "long"),
-                axis=1,
-                meta=meta,
+            df["clean_code_tup"] = df[long_col].map_partitions(
+                lambda srs: [_format_lat_or_long(x, output_format, errors, "long") for x in srs],
+                meta=object,
             )
+            df = df.assign(
+                _temp_=df["clean_code_tup"].map(itemgetter(0)),
+                _code_=df["clean_code_tup"].map(itemgetter(1)),
+            )
+            df = df.rename(columns={"_temp_": f"{long_col}_clean"})
             if inplace:
-                df = df.drop(columns=[long_col])
+                df = df.drop(columns=long_col)
         # merge the cleaned latitude and longitude
         if lat_col and long_col and not split:
             if output_format == "dd":
                 df["latitude_longitude"] = df[[f"{lat_col}_clean", f"{long_col}_clean"]].apply(
-                    tuple, axis=1
+                    tuple, axis=1, meta=object
                 )
             else:
                 df["latitude_longitude"] = df[f"{lat_col}_clean"] + ", " + df[f"{long_col}_clean"]
+
+            # if seperate lat and long columns are merged, then all values are "cleaned"
+            df["_code_"] = 2
             df = df.drop(columns=[f"{lat_col}_clean", f"{long_col}_clean"])
 
-    df, nrows = dask.compute(df, df.shape[0])
+    # counts of codes indicating how values were changed
+    stats = df["_code_"].value_counts(sort=False)
+    df = df.drop(columns=["clean_code_tup", "_code_"])
 
-    # output the report describing the changes to the column
+    with ProgressBar(minimum=1, disable=not progress):
+        df, stats = dask.compute(df, stats)
+
+    # output a report describing the result of clean_lat_long
     if report:
-        create_report("Latitude and Longitude", STATS, nrows)
+        create_report_new("Latitude and Longitude", stats, errors)
 
     return df
 
@@ -222,57 +240,49 @@ def validate_lat_long(
     if lat or lon:
         hor_dir = "lat" if lat else "long"
         if isinstance(x, pd.Series):
-            return x.apply(check_lat_or_long, args=(False, hor_dir))
-        else:
-            return check_lat_or_long(x, False, hor_dir)
+            return x.apply(_check_lat_or_long, args=(False, hor_dir))
+        return _check_lat_or_long(x, False, hor_dir)
     elif lat_long:
         if isinstance(x, pd.Series):
-            return x.apply(check_lat_long, args=(False,))
-        else:
-            return check_lat_long(x, False)
+            return x.apply(_check_lat_long, args=(False,))
+        return _check_lat_long(x, False)
 
     return None
 
 
-def format_lat_long(
-    row: pd.Series,
-    col: str,
+def _format_lat_long(
+    val: Any,
     output_format: str,
     split: bool,
     errors: str,
-) -> pd.Series:
+) -> Any:
     """
-    Function to transform a coordinate instance into the
-    desired format
+    Function to transform a coordinate instance into the desired format
+
+    The last component of the returned tuple contains a code indicating how the
+    input value was changed:
+        0 := the value is null
+        1 := the value could not be parsed
+        2 := the value is cleaned and the cleaned value is DIFFERENT than the input value
+        3 := the value is cleaned and is THE SAME as the input value (no transformation)
     """
-    # pylint: disable=too-many-locals,too-many-branches,too-many-statements
-    # check_lat_long parses the value in row[col], and will return the components
-    # if the parse is succesful. The returned value "status" can be either "null"
-    # (which means row[col] contains a null value), "unknown" (in which case the value
-    # in row[col] could not be parsed) or "success" (a succesful parse of the value).
+    # pylint: disable=too-many-locals
+    # _check_lat_long parses the value val, and will return the components
+    # if the parse is succesful. The returned value "status" can be either 0 ie
+    # "null" (which means val is a null value), 1 ie ("unkwonw") (in which case
+    # val could not be parsed) or 2 ie "success" (a succesful parse of val).
     # dds, mins, secs, hem are the latitude components and # dds2, mins2, secs2, hem2
     # are the longitude components
-    dds, mins, secs, hem, dds2, mins2, secs2, hem2, status = check_lat_long(row[col], True)
+    dds, mins, secs, hem, dds2, mins2, secs2, hem2, status = _check_lat_long(val, True)
 
-    if status == "null":  # row[col] contains a null value
-        STATS["null"] += 1
-        if split:
-            row["latitude"], row["longitude"] = np.nan, np.nan
-        else:
-            row[f"{col}_clean"] = np.nan
-        return row
+    if status == 0:  # val is a null value
+        return (np.nan, np.nan, 0) if split else (np.nan, 0)
 
-    if status == "unknown":  # row[col] contains an unknown value
+    if status == 1:  # val contains an unknown value
         if errors == "raise":
-            raise ValueError(f"unable to parse value {row[col]}")
-
-        STATS["unknown"] += 1
-        if split:
-            row["latitude"] = row[col] if errors == "ignore" else np.nan
-            row["longitude"] = np.nan
-        else:
-            row[f"{col}_clean"] = row[col] if errors == "ignore" else np.nan
-        return row
+            raise ValueError(f"unable to parse value {val}")
+        result = val if errors == "ignore" else np.nan
+        return (result, np.nan, 1) if split else (result, 1)
 
     # derive the hemisphere if not given in the initial coordinate
     if not hem:
@@ -307,33 +317,29 @@ def format_lat_long(
         secs2 = int(secs2) if secs2.is_integer() else secs2
         lat = f"{int(dds)}{chr(176)} {mins}{chr(8242)} {secs}{chr(8243)} {hem}"
         lon = f"{int(dds2)}{chr(176)} {mins2}{chr(8242)} {secs2}{chr(8243)} {hem2}"
+
     if split:
-        STATS["cleaned"] += 1
-        row["latitude"], row["longitude"] = lat, lon
-    else:
-        row[f"{col}_clean"] = (lat, lon) if output_format == "dd" else f"{lat}, {lon}"
-        if row[col] != row[f"{col}_clean"]:
-            STATS["cleaned"] += 1
-
-    return row
+        return lat, lon, 2
+    result = (lat, lon) if output_format == "dd" else f"{lat}, {lon}"
+    return result, 2 if val != result else 3
 
 
-def check_lat_long(val: Union[str, float, Any], clean: bool) -> Any:
+def _check_lat_long(val: Any, clean: bool) -> Any:
     """
-    Function to transform a coordinate instance into the
-    desired format
+    Function to check if a coordinate instance is valid
     """
+    # pylint: disable=too-many-boolean-expressions
     # if the value is null, return empty strings for the components
-    # and "null" for the "status"
+    # and the code 0 to indicate a null status
     if val in NULL_VALUES:
-        return [""] * 8 + ["null"] if clean else False
+        return (None,) * 8 + (0,) if clean else False
 
     mch = re.match(LAT_LONG_PATTERN, re.sub(r"''", r'"', str(val)))
     # check if the value was able to be parsed
     if not mch:
-        return [""] * 8 + ["unknown"] if clean else False
+        return (None,) * 8 + (1,) if clean else False
     if not mch.group("deg") or not mch.group("deg2"):
-        return [""] * 8 + ["unknown"] if clean else False
+        return (None,) * 8 + (1,) if clean else False
 
     # coordinates for latitude
     mins = float(mch.group("min")) if mch.group("min") else 0
@@ -349,7 +355,7 @@ def check_lat_long(val: Union[str, float, Any], clean: bool) -> Any:
 
     # minutes and seconds need to be in the interval [0, 60)
     # for degrees:
-    #  if hemisphere is give, then 0<=lat<=90 and 0<=long<=180
+    #  if hemisphere is given, then 0<=lat<=90 and 0<=long<=180
     #  if hemisphere is not given, then -90<=lat<=90 and -180<=long<=180
     # decimal degrees must be -90<=lat<=90 and -180<=long<=180
     # the first given hemisphere and last hemisphere cannot both be set
@@ -371,31 +377,24 @@ def check_lat_long(val: Union[str, float, Any], clean: bool) -> Any:
         or sum([mch.group("dir_back") is not None, mch.group("dir_front") is not None]) > 1
         or sum([mch.group("dir_back2") is not None, mch.group("dir_front2") is not None]) > 1
     ):
-        return [""] * 8 + ["unknown"] if clean else False
+        return (None,) * 8 + (1,) if clean else False
 
-    return (dds, mins, secs, hem, dds2, mins2, secs2, hem2, "success") if clean else True
+    return (dds, mins, secs, hem, dds2, mins2, secs2, hem2, 2) if clean else True
 
 
-def format_lat_or_long(
-    row: pd.Series, col: str, output_format: str, errors: str, hor_dir: str
-) -> pd.Series:
+def _format_lat_or_long(val: Any, output_format: str, errors: str, hor_dir: str) -> Any:
     """
-    Function to transform a coordinate instance into the
-    desired format
+    Function to transform a coordinate instance into the desired format
     """
-    dds, mins, secs, hem, status = check_lat_or_long(row[col], True, hor_dir)
+    dds, mins, secs, hem, status = _check_lat_or_long(val, True, hor_dir)
 
-    if status == "null":
-        STATS["null"] += 1
-        row[f"{col}_clean"] = np.nan
-        return row
+    if status == 0:  # val contains a null value
+        return np.nan, 0
 
-    if status == "unknown":
+    if status == 1:  # val contains an unknown value
         if errors == "raise":
-            raise ValueError(f"unable to parse value {row[col]}")
-        STATS["unknown"] += 1
-        row[f"{col}_clean"] = row[col] if errors == "ignore" else np.nan
-        return row
+            raise ValueError(f"unable to parse value {val}")
+        return val if errors == "ignore" else np.nan, 1
 
     if not hem:
         if hor_dir == "lat":
@@ -419,28 +418,24 @@ def format_lat_or_long(
         secs = int(secs) if secs.is_integer() else secs
         res = f"{int(dds)}{chr(176)} {mins}{chr(8242)} {secs}{chr(8243)} {hem}"
 
-    row[f"{col}_clean"] = res
-    if row[col] != row[f"{col}_clean"]:
-        STATS["cleaned"] += 1
-
-    return row
+    return res, 2 if val != res else 3
 
 
-def check_lat_or_long(val: Union[str, float, Any], clean: bool, hor_dir: str) -> Any:
+def _check_lat_or_long(val: Any, clean: bool, hor_dir: str) -> Any:
     """
-    Function to transform a coordinate instance into the
-    desired format
+    Function to check if a coordinate instance is valid
     """
+    # pylint: disable=too-many-boolean-expressions
     if val in NULL_VALUES:
-        return [""] * 4 + ["null"] if clean else False
+        return (None,) * 4 + (0,) if clean else False
 
     pat = LAT_PATTERN if hor_dir == "lat" else LONG_PATTERN
 
     mch = re.match(pat, re.sub(r"''", r'"', str(val)))
     if not mch:
-        return [""] * 4 + ["unknown"] if clean else False
+        return (None,) * 4 + (1,) if clean else False
     if not mch.group("deg"):
-        return [""] * 4 + ["unknown"] if clean else False
+        return (None,) * 4 + (1,) if clean else False
 
     # coordinates
     mins = float(mch.group("min")) if mch.group("min") else 0
@@ -467,15 +462,6 @@ def check_lat_or_long(val: Union[str, float, Any], clean: bool, hor_dir: str) ->
         or abs(dds) > bound
         or sum([mch.group("dir_back") is not None, mch.group("dir_front") is not None]) > 1
     ):
-        return [""] * 4 + ["unknown"] if clean else False
+        return (None,) * 4 + (1,) if clean else False
 
-    return (dds, mins, secs, hem, "success") if clean else True
-
-
-def reset_stats() -> None:
-    """
-    Reset global statistics dictionary
-    """
-    STATS["cleaned"] = 0
-    STATS["null"] = 0
-    STATS["unknown"] = 0
+    return (dds, mins, secs, hem, 2) if clean else True
