@@ -5,22 +5,22 @@ Every data fetching action should begin with instantiating this Connector class.
 import math
 import sys
 from asyncio import as_completed
-from pathlib import Path
-from typing import Any, Awaitable, Dict, List, Optional, Tuple, Union
+from typing import Any, Awaitable, Dict, Optional, Set, Tuple, Union
 from warnings import warn
 
 import pandas as pd
 from aiohttp import ClientSession
 from aiohttp.client_reqrep import ClientResponse
-from jinja2 import Environment, StrictUndefined, Template, UndefinedError
+from jinja2 import Environment, StrictUndefined, UndefinedError
 from jsonpath_ng import parse as jparse
 
-from .config_manager import config_directory, ensure_config
 from .errors import InvalidParameterError, RequestError, UniversalParameterOverridden
 from .implicit_database import ImplicitDatabase, ImplicitTable
+from .info import info
+from .config_manager import initialize_path
 from .ref import Ref
 from .schema import (
-    ConfigDef,
+    FieldDef,
     FieldDefUnion,
     OffsetPaginationDef,
     PagePaginationDef,
@@ -30,25 +30,8 @@ from .schema import (
 )
 from .throttler import OrderedThrottler, ThrottleSession
 
-INFO_TEMPLATE = Template(
-    """{% for tb in tbs.keys() %}
-Table {{dbname}}.{{tb}}
 
-Parameters
-----------
-{% if tbs[tb].required_params %}{{", ".join(tbs[tb].required_params)}} required {% endif %}
-{% if tbs[tb].optional_params %}{{", ".join(tbs[tb].optional_params)}} optional {% endif %}
-
-Examples
---------
->>> dc.query({{", ".join(["\\\"{}\\\"".format(tb)] + tbs[tb].joined_query_fields)}})
->>> dc.show_schema("{{tb}}")
-{% endfor %}
-"""
-)
-
-
-class Connector:
+class Connector:  # pylint: disable=too-many-instance-attributes
     """This is the main class of the connector component.
     Initialize Connector class as the example code.
 
@@ -79,6 +62,7 @@ class Connector:
     # storage for authorization
     _storage: Dict[str, Any]
     _concurrency: int
+    _update: bool
     _jenv: Environment
 
     def __init__(
@@ -90,16 +74,7 @@ class Connector:
         _concurrency: int = 1,
         **kwargs: Any,
     ) -> None:
-        if (
-            config_path.startswith(".")
-            or config_path.startswith("/")
-            or config_path.startswith("~")
-        ):
-            path = Path(config_path).resolve()
-        else:
-            # From Github!
-            ensure_config(config_path, update)
-            path = config_directory() / config_path
+        path = initialize_path(config_path, update)
 
         self._impdb = ImplicitDatabase(path)
 
@@ -107,6 +82,7 @@ class Connector:
         self._auth = _auth or {}
         self._storage = {}
         self._concurrency = _concurrency
+        self._update = update
         self._jenv = Environment(undefined=StrictUndefined)
         self._throttler = OrderedThrottler(_concurrency)
 
@@ -138,81 +114,29 @@ class Connector:
         **where
             The additional parameters required for the query.
         """
-        allowed_params = self._impdb.tables[table].config.request.params
+        allowed_params: Set[str] = set()
+
+        for key, val in self._impdb.tables[table].config.request.params.items():
+            if isinstance(val, FieldDef):
+                if isinstance(val.from_key, list):
+                    allowed_params.update(val.from_key)
+                elif isinstance(val.from_key, str):
+                    allowed_params.add(val.from_key)
+                else:
+                    allowed_params.add(key)
+            else:
+                allowed_params.add(key)
+
         for key in where:
             if key not in allowed_params:
                 raise InvalidParameterError(key)
 
         return await self._query_imp(table, where, _auth=_auth, _q=_q, _count=_count)
 
-    @property
-    def table_names(self) -> List[str]:
-        """
-        Return all the names of the available tables in a list.
-
-        Note
-        ----
-        We abstract each website as a database containing several tables.
-        For example in Spotify, we have artist and album table.
-        """
-        return list(self._impdb.tables.keys())
-
     def info(self) -> None:
         """Show the basic information and provide guidance for users
         to issue queries."""
-
-        # get info
-        tbs: Dict[str, Any] = {}
-        for cur_table in self._impdb.tables:
-            table_config_content: ConfigDef = self._impdb.tables[cur_table].config
-            params_required = []
-            params_optional = []
-            example_query_fields = []
-            count = 1
-            for k, val in table_config_content.request.params.items():
-                if isinstance(val, bool) and val:
-                    params_required.append(k)
-                    example_query_fields.append(f"""{k}="word{count}\"""")
-                    count += 1
-                elif isinstance(val, bool):
-                    params_optional.append(k)
-            tbs[cur_table] = {}
-            tbs[cur_table]["required_params"] = params_required
-            tbs[cur_table]["optional_params"] = params_optional
-            tbs[cur_table]["joined_query_fields"] = example_query_fields
-
-        # show table info
-        print(INFO_TEMPLATE.render(ntables=len(self.table_names), dbname=self._impdb.name, tbs=tbs))
-
-    def show_schema(self, table_name: str) -> pd.DataFrame:
-        """This method shows the schema of the table that will be returned,
-        so that the user knows what information to expect.
-
-        Parameters
-        ----------
-        table_name
-            The table name.
-
-        Returns
-        -------
-        pd.DataFrame
-            The returned data's schema.
-
-        Note
-        ----
-        The schema is defined in the configuration file.
-        The user can either use the default one or change it by editing the configuration file.
-        """
-        print(f"table: {table_name}")
-        table_config_content: ConfigDef = self._impdb.tables[table_name].config
-        schema = table_config_content.response.schema_
-        new_schema_dict: Dict[str, List[Any]] = {}
-        new_schema_dict["column_name"] = []
-        new_schema_dict["data_type"] = []
-        for k in schema.keys():
-            new_schema_dict["column_name"].append(k)
-            new_schema_dict["data_type"].append(schema[k].type)
-        return pd.DataFrame.from_dict(new_schema_dict)
+        info(self._impdb.name)
 
     async def _query_imp(  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
         self,
@@ -489,20 +413,19 @@ def validate_fields(fields: Dict[str, FieldDefUnion], data: Dict[str, Any]) -> N
     """Check required fields are provided."""
 
     for key, def_ in fields.items():
-        from_key, to_key = key, key
+        to_key = key
 
         if isinstance(def_, bool):
             required = def_
             if required and to_key not in data:
-                raise KeyError(f"'{from_key}' is required but not provided")
+                raise KeyError(f"'{to_key}' is required but not provided")
         elif isinstance(def_, str):
             pass
         else:
             to_key = def_.to_key or to_key
-            from_key = def_.from_key or from_key
             required = def_.required
             if required and to_key not in data:
-                raise KeyError(f"'{from_key}' is required but not provided")
+                raise KeyError(f"'{to_key}' is required but not provided")
 
 
 def populate_field(  # pylint: disable=too-many-branches
@@ -514,10 +437,10 @@ def populate_field(  # pylint: disable=too-many-branches
     ret: Dict[str, str] = {}
 
     for key, def_ in fields.items():
-        from_key, to_key = key, key
+        to_key = key
 
         if isinstance(def_, bool):
-            value = params.get(from_key)
+            value = params.get(to_key)
             remove_if_empty = False
         elif isinstance(def_, str):
             # is a template
@@ -528,10 +451,9 @@ def populate_field(  # pylint: disable=too-many-branches
             template = def_.template
             remove_if_empty = def_.remove_if_empty
             to_key = def_.to_key or to_key
-            from_key = def_.from_key or from_key
 
             if template is None:
-                value = params.get(from_key)
+                value = params.get(to_key)
             else:
                 tmplt = jenv.from_string(template)
                 try:
