@@ -1,6 +1,8 @@
 """Strong typed schema definition."""
 import http.server
+import json
 import random
+import socket
 import socketserver
 import string
 from base64 import b64encode
@@ -8,13 +10,15 @@ from enum import Enum
 from pathlib import Path
 from threading import Thread
 from time import time
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Set, Union
 from urllib.parse import parse_qs, urlparse
-import socket
-import requests
-from pydantic import Field
+
+from jinja2 import Environment, meta
+from pydantic import Field, root_validator
 
 from ...utils import is_notebook
+from ..errors import InvalidAuthParams, MissingRequiredAuthParams
+from ..utils import Request
 from .base import BaseDef, BaseDefT
 
 # pylint: disable=missing-class-docstring,missing-function-docstring
@@ -30,6 +34,16 @@ def get_random_string(length: int) -> str:
     return result_str
 
 
+def validate_auth(required: Set[str], passed: Dict[str, Any]) -> None:
+    required_not_passed = required - passed.keys()
+    passed_not_required = passed.keys() - required
+    if required_not_passed:
+        raise MissingRequiredAuthParams(required_not_passed)
+
+    if passed_not_required:
+        raise InvalidAuthParams(passed_not_required)
+
+
 class OffsetPaginationDef(BaseDef):
     type: str = Field("offset", const=True)
     max_count: int
@@ -71,11 +85,36 @@ PaginationDef = Union[OffsetPaginationDef, SeekPaginationDef, PagePaginationDef,
 
 class FieldDef(BaseDef):
     required: bool
-    from_key: Optional[str]
+    from_key: Union[List[str], str, None]
     to_key: Optional[str]
     template: Optional[str]
     remove_if_empty: bool
 
+    @root_validator(pre=True)
+    # pylint: disable=no-self-argument,no-self-use
+    def from_key_validation(cls, values: Dict[str, Any]) -> Any:
+        if "template" in values:
+            parsed_content = Environment().parse(values["template"])
+            variables = meta.find_undeclared_variables(parsed_content)  # type: ignore
+
+            from_key = values.get("fromKey")
+            if isinstance(from_key, str):
+                from_key = {from_key}
+            elif from_key is None:
+                from_key = set()
+            elif isinstance(from_key, list):
+                from_key = set(from_key)
+            else:
+                raise NotImplementedError("Unreachable")
+
+            if len(set(variables) - from_key) != 0:
+                raise ValueError(f"template requires {variables} exist in fromKey, got {from_key}")
+        else:
+            if isinstance(values.get("fromKey"), list):
+                raise ValueError("from_key cannot be a list if template is not used.")
+
+        return values
+
 
 FieldDefUnion = Union[FieldDef, bool, str]  # Put bool before str
 
@@ -130,19 +169,23 @@ class OAuth2AuthorizationCodeAuthorizationDef(BaseDef):
             port = params.get("port", 9999)
             code = self._auth(params["client_id"], port)
 
+            validate_auth({"client_id", "client_secret"}, params)
             ckey = params["client_id"]
             csecret = params["client_secret"]
             b64cred = b64encode(f"{ckey}:{csecret}".encode("ascii")).decode()
 
-            resp: Dict[str, Any] = requests.post(
-                self.token_server_url,
-                headers={"Authorization": f"Basic {b64cred}"},
-                data={
-                    "grant_type": "authorization_code",
-                    "code": code,
-                    "redirect_uri": f"http://localhost:{port}/",
-                },
-            ).json()
+            headers = {
+                "Authorization": f"Basic {b64cred}",
+                "Content-Type": "application/x-www-form-urlencoded",
+            }
+            params = {
+                "grant_type": "authorization_code",
+                "code": code,
+                "redirect_uri": f"http://localhost:{port}/",
+            }
+            requests = Request(self.token_server_url)
+            response = requests.post(_headers=headers, _data=params)
+            resp: Dict[str, Any] = json.loads(response.read())
 
             if resp["token_type"].lower() != "bearer":
                 raise RuntimeError("token_type is not bearer")
@@ -208,15 +251,22 @@ class OAuth2ClientCredentialsAuthorizationDef(BaseDef):
             raise ValueError("storage is required for OAuth2")
 
         if "access_token" not in storage or storage.get("expires_at", 0) < time():
+            validate_auth({"client_id", "client_secret"}, params)
+
             # Not yet authorized
             ckey = params["client_id"]
             csecret = params["client_secret"]
             b64cred = b64encode(f"{ckey}:{csecret}".encode("ascii")).decode()
-            resp: Dict[str, Any] = requests.post(
-                self.token_server_url,
-                headers={"Authorization": f"Basic {b64cred}"},
-                data={"grant_type": "client_credentials"},
-            ).json()
+
+            headers = {
+                "Authorization": f"Basic {b64cred}",
+                "Content-Type": "application/x-www-form-urlencoded",
+            }
+            params = {"grant_type": "client_credentials"}
+            requests = Request(self.token_server_url)
+            response = requests.post(_headers=headers, _data=params)
+            resp: Dict[str, Any] = json.loads(response.read())
+
             if resp["token_type"].lower() != "bearer":
                 raise RuntimeError("token_type is not bearer")
 
@@ -242,6 +292,8 @@ class QueryParamAuthorizationDef(BaseDef):
     ) -> None:
         """Populate some required fields to the request data."""
 
+        validate_auth({"access_token"}, params)
+
         req_data["params"][self.key_param] = params["access_token"]
 
 
@@ -256,6 +308,8 @@ class BearerAuthorizationDef(BaseDef):
     ) -> None:
         """Populate some required fields to the request data."""
 
+        validate_auth({"access_token"}, params)
+
         req_data["headers"]["Authorization"] = f"Bearer {params['access_token']}"
 
 
@@ -272,6 +326,8 @@ class HeaderAuthorizationDef(BaseDef):
     ) -> None:
         """Populate some required fields to the request data."""
 
+        validate_auth({"access_token"}, params)
+
         req_data["headers"][self.key_name] = params["access_token"]
         req_data["headers"].update(self.extra)
 
@@ -379,4 +435,5 @@ class ResponseDef(BaseDef):
 class ConfigDef(BaseDef):
     version: int = Field(1, const=True)
     request: RequestDef
+    examples: Optional[Dict[str, str]]
     response: ResponseDef
