@@ -19,6 +19,25 @@ ENV_LOADER = Environment(
     loader=PackageLoader("dataprep", "eda/templates"),
 )
 
+TAB_VISUAL_TYPES = {
+    "missing_impact_1v1",
+    "missing_impact",
+    "categorical_column",
+    "numerical_column",
+    "datetime_column",
+    "cat_and_num_cols",
+    "two_num_cols",
+    "two_cat_cols",
+    "dt_and_num_cols",
+    "dt_and_cat_cols",
+    "dt_cat_num_cols",
+    "correlation_impact",
+    "correlation_single_heatmaps",
+    "correlation_scatter",
+}
+
+GRID_VISUAL_TYPES = {"distribution_grid", "missing_impact_1vn"}
+
 
 class Container:
     """
@@ -30,38 +49,17 @@ class Container:
         to_render: Dict[str, Any],
         visual_type: str,
     ) -> None:
-        if visual_type in ("distribution_grid", "missing_impact_1vn"):
-            self.context = {
-                "resources": INLINE.render(),
-                "components": components(to_render.get("layout")),
-                "tabledata": to_render.get("tabledata"),
-                "overview_insights": to_render.get("overview_insights"),
-                "column_insights": to_render.get("column_insights"),
-                "meta": to_render.get("meta"),
-                "title": "DataPrep.EDA Report",
-                "rnd": random.randint(0, 99),  # for multiple cells running in the same notebook
-                "container_width": f"{to_render['fig_width']*3}px",
-                "legend_labels": to_render.get("legend_labels"),
-            }
+        self.context = Context(**to_render)
+        setattr(self.context, "rnd", random.randint(0, 9999))
+        if visual_type in GRID_VISUAL_TYPES:
             self.template_base = ENV_LOADER.get_template("grid_base.html")
-
-        elif "_column" in visual_type or visual_type in (
-            "missing_impact",
-            "missing_impact_1v1",
-        ):
-            # todo: param management
+        elif visual_type in TAB_VISUAL_TYPES:
             if to_render.get("tabledata"):
-                to_render["meta"].insert(0, "Stats")
-            self.context = {
-                "resources": INLINE.render(),
-                "tabledata": to_render.get("tabledata"),
-                "insights": to_render.get("insights"),
-                "components": components(to_render.get("layout")),
-                "meta": to_render.get("meta"),
-                "title": "DataPrep.EDA Report",
-                "rnd": random.randint(100, 999),  # for multiple cells running in the same notebook
-            }
-            self.template_base = ENV_LOADER.get_template("tab_base.html")
+                self.context.meta.insert(0, "Stats")  # type: ignore
+            if visual_type == "correlation_impact":
+                self.template_base = ENV_LOADER.get_template("tab_base_corr.html")
+            else:
+                self.template_base = ENV_LOADER.get_template("tab_base.html")
         else:
             raise TypeError(f"Unsupported Visual Type: {visual_type}.")
 
@@ -69,7 +67,7 @@ class Container:
         """
         save function
         """
-        with open(filename, "w") as f:
+        with open(filename, "w", encoding="utf-8") as f:
             f.write(self.template_base.render(context=self.context))
 
     def _repr_html_(self) -> str:
@@ -109,6 +107,37 @@ class Container:
         # set delete = False to avoid early delete when user open multiple plots.
         with NamedTemporaryFile(suffix=".html", delete=False) as tmpf:
             pass
-        with open(tmpf.name, "w") as file:
+        with open(tmpf.name, "w", encoding="utf-8") as file:
             file.write(self.template_base.render(context=self.context))
         webbrowser.open_new_tab(f"file://{tmpf.name}")
+
+
+class Context:
+    """
+    Define the context class that stores all the parameters needed by template engine.
+    The instance is read-only.
+    """
+
+    _title = "DataPrep.EDA Report"
+    _resources = INLINE.render()
+    _container_width = 650  # default width just in case nothing got passed in
+
+    def __init__(self, **param: Any) -> None:
+        self.title = self._title
+        self.resources = self._resources
+        self.container_width = self._container_width
+
+        for attr, value in param.items():
+            if attr == "layout":
+                setattr(self, "components", components(value))
+            else:
+                setattr(self, attr, value)
+
+    def __getitem__(self, key: str) -> Any:
+        try:
+            return getattr(self, key)
+        except KeyError:
+            return None
+
+    def __getattr__(self, attr: str) -> None:
+        return None
