@@ -37,8 +37,8 @@ __all__ = ["render_missing"]
 
 def render_missing(
     itmdt: Intermediate,
-    plot_width: int = 500,
-    plot_height: int = 500,
+    plot_width: int = 400,
+    plot_height: int = 400,
 ) -> Dict[str, Any]:
     """
     @Jinglin write here
@@ -46,7 +46,7 @@ def render_missing(
     if itmdt.visual_type == "missing_impact":
         return render_missing_impact(itmdt, plot_width, plot_height)
     elif itmdt.visual_type == "missing_impact_1vn":
-        return render_missing_impact_1vn(itmdt, plot_width - 176, plot_height - 200)
+        return render_missing_impact_1vn(itmdt, plot_width - 100, plot_height - 100)
     elif itmdt.visual_type == "missing_impact_1v1":
         return render_missing_impact_1v1(itmdt, plot_width, plot_height)
     else:
@@ -121,7 +121,7 @@ def render_dist(
             legend_label=label,
         )
 
-    relocate_legend(fig, "right")
+    relocate_legend(fig, "left")
 
     return fig
 
@@ -191,7 +191,7 @@ def render_hist(  # pylint: disable=too-many-arguments
             legend_field="label",
         )
 
-        relocate_legend(fig, "right")
+        relocate_legend(fig, "left")
     else:
         shown, total = meta["partial"]
         if shown != total:
@@ -310,9 +310,7 @@ def create_color_mapper_heatmap(
     return mapper, colorbar
 
 
-def render_missing_impact(
-    itmdt: Intermediate, plot_width: int, plot_height: int
-) -> Dict[str, List[Any]]:
+def render_missing_impact(itmdt: Intermediate, plot_width: int, plot_height: int) -> Dict[str, Any]:
     """
     Render correlation heatmaps in to tabs
     """
@@ -331,9 +329,14 @@ def render_missing_impact(
     fig_dendrogram = render_dendrogram(itmdt["data_dendrogram"], plot_width, plot_height)
     tabs.append(Panel(child=row(fig_dendrogram), title="Dendrogram"))
 
+    stat_dict = {name: itmdt["missing_stat"][name] for name in itmdt["missing_stat"]}
+
     return {
+        "insights": itmdt["insights"],
+        "tabledata": {"Missing Statistics": stat_dict},
         "layout": [panel.child.children[0] for panel in tabs],
         "meta": [panel.title for panel in tabs],
+        "container_width": plot_width + 160,
     }
 
 
@@ -390,6 +393,7 @@ def render_heatmaps(df: Optional[pd.DataFrame], plot_width: int, plot_height: in
                 toolbar_location=None,
                 tooltips=tooltips,
                 background_fill_color="#fafafa",
+                title=" ",
             )
 
             fig.rect(
@@ -409,8 +413,8 @@ def render_heatmaps(df: Optional[pd.DataFrame], plot_width: int, plot_height: in
     tweak_figure(fig)
     fig.grid.grid_line_color = None
     fig.axis.axis_line_color = None
-    fig.add_layout(color_bar, "right")
-
+    fig.add_layout(color_bar, "left")
+    fig.frame_width = plot_width
     return fig
 
 
@@ -437,6 +441,7 @@ def render_bar_chart(
         y_axis_type=yscale,
         toolbar_location=None,
         tools=[],
+        title=" ",
     )
 
     rend = fig.vbar_stack(
@@ -478,7 +483,8 @@ def render_bar_chart(
 
     fig.yaxis.axis_label = "Row Count"
     tweak_figure(fig)
-    relocate_legend(fig, "right")
+    relocate_legend(fig, "left")
+    fig.frame_width = plot_width
 
     return fig
 
@@ -528,6 +534,7 @@ def render_missing_spectrum(
             tools="hover",
             toolbar_location=None,
             tooltips=tooltips,
+            title=" ",
         )
     )
     fig.xgrid.grid_line_color = None
@@ -543,7 +550,8 @@ def render_missing_spectrum(
         fill_color={"field": "missing_rate", "transform": mapper},
         line_color=None,
     )
-    fig.add_layout(color_bar, "right")
+    fig.add_layout(color_bar, "left")
+    fig.frame_width = plot_width
     return fig
 
 
@@ -563,6 +571,7 @@ def render_dendrogram(dend: Dict["str", Any], plot_width: int, plot_height: int)
         plot_height=plot_height,
         toolbar_location=None,
         tools="",
+        title=" ",
     )
 
     # round the coordinates to integers, and plot the dendrogram
@@ -592,7 +601,7 @@ def render_dendrogram(dend: Dict["str", Any], plot_width: int, plot_height: int)
     fig.xaxis.major_label_orientation = np.pi / 3
     fig.yaxis.axis_label = "Average Distance Between Clusters"
     fig.grid.visible = False
-
+    fig.frame_width = plot_width
     return fig
 
 
@@ -617,7 +626,7 @@ def render_missing_impact_1vn(
     legend_colors = [CATEGORY10[count] for count in range(len(LABELS))]
     return {
         "layout": [panel.child for panel in panels],
-        "fig_width": plot_width,
+        "container_width": plot_width * 3,
         "legend_labels": [
             {"label": label, "color": color} for label, color in zip(LABELS, legend_colors)
         ],
@@ -628,7 +637,7 @@ def render_missing_impact_1v1(
     itmdt: Intermediate,
     plot_width: int,
     plot_height: int,
-) -> Dict[str, List[Any]]:
+) -> Dict[str, Any]:
     """
     Render the plot from `plot_missing(df, "x", "y")`
     """
@@ -650,16 +659,20 @@ def render_missing_impact_1v1(
         fig = render_boxwhisker(itmdt["box"], plot_width, plot_height)
         panels.append(Panel(child=fig, title="Box"))
 
+        for panel in panels:
+            panel.child.frame_width = plot_width
+
         return {
             "layout": [panel.child for panel in panels],
             "meta": [panel.title for panel in panels],
+            "container_width": plot_width + 240,
         }
     else:
         fig = render_hist(itmdt["hist"], y, meta, plot_width, plot_height, True)
-
+        fig.frame_width = plot_width
         shown, total = meta["partial"]
         if shown != total:
-            fig.title = Title(text=f"Missing impact of {x} by ({shown} out of {total}) {y}")
+            _title = f"Missing impact of {x} by ({shown} out of {total}) {y}"
         else:
-            fig.title = Title(text=f"Missing impact of {x} by {y}")
-        return {"layout": [fig], "meta": [fig.title.text]}
+            _title = f"Missing impact of {x} by {y}"
+        return {"layout": [fig], "meta": [_title], "container_width": plot_width + 240}
